"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var ethers_1 = require("ethers");
var ensjs_1 = require("@ensdomains/ensjs");
var contracts_1 = require("@orcaprotocol/contracts");
var axios_1 = __importDefault(require("axios"));
var pod_utils_1 = require("./pod-utils");
var config_1 = require("./config");
var fetchers_1 = require("./fetchers");
var utils_1 = require("./lib/utils");
var transaction_service_1 = require("./lib/services/transaction-service");
var create_safe_transaction_1 = require("./lib/services/create-safe-transaction");
var Proposal_1 = __importDefault(require("./Proposal"));
var subgraph_1 = require("./lib/services/subgraph");
/**
 * The `Pod` object is the interface for fetching pod data.
 *
 * The Pod object should not be instantiated directly, use {@link getPod} instead.
 *
 * The following properties are on the object itself:
 *
 * ```js
 * const {
 *   id, // Pod ID
 *   safe, // Gnosis safe address, aka the Pod address
 *   ensName, // E.g., orcanauts.pod.xyz
 *   admin, // Address of pod admin
 *   imageUrl, // Source of NFT image
 *   imageNoTextUrl, // Source of NFT image without text (used for avatars)
 * } = await getPod();
 * ```
 *
 * Members, EOAs and member Pods can be fetched with the following functions:
 *
 * ```js
 * const pod = await getPod(podAddress);
 * // Fetches list of all members from the pod, as an array of Ethereum addresses.
 * // This includes any pods that may be members of the original pods.
 * const members = await pod.getMembers();
 *
 * // Fetches any member EOAs (externally owned accounts). That is, any member that is not a smart contract or pod.
 * const memberEOAs = await pod.getMemberEOAs();
 *
 * // Fetches Pod objects for any member pods.
 * const memberPods = await pod.getMemberPods();
 * ```
 *
 * You can also check if a user is a member, admin, or member of those pods with the following functions:
 *
 * ```js
 * const pod = await getPod(podAddress);
 *
 * const isMember = await pod.isMember(userAddress);
 * // Not an async function
 * const isAdmin = pod.isAdmin(userAddress);
 *
 * const isAdminPodMember = await pod.isAdminPodMember(userAddress);
 *
 * // Includes both pods and users as sub pod members.
 * const isSubPodMember = await pod.isSubPodMember(userAddress);
 * ```
 */
var Pod = /** @class */ (function () {
    /**
     *  Note this constructor should not be called directly. Use `getPod()` instead.
     * @param identifier Can be either podId or safe address
     */
    function Pod(identifier) {
        var _this = this;
        /**
         * Returns an array of Proposal objects in reverse chronological order. Defaults to returning 5,
         * which can be overridden by passing { limit: 10 } for example in the options.
         *
         * By default, the first Proposal will be the active proposal, if there is one, and then any executed proposals.
         *
         * Queued proposals can be fetched by passing { status: 'queued' } in the options. This will return queued
         * proposals, then the active proposal, then executed proposals (up to the requested limit).
         *
         * Executed proposals can be fetched by passing { status: 'passed' } or { status: 'rejected' } in the options. This will return
         * only executed proposals.
         *
         * @param options
         * @returns
         */
        this.getProposals = function (options) {
            if (options === void 0) { options = {}; }
            return __awaiter(_this, void 0, void 0, function () {
                var _a, nonce, threshold, _b, limit, params, safeTransactions, _c, _d, normalTransactions, rejectTransactions, pairedSubTxs, rejectNonces, subProposals, normalProposals;
                var _this = this;
                return __generator(this, function (_e) {
                    switch (_e.label) {
                        case 0:
                            _a = this, nonce = _a.nonce, threshold = _a.threshold;
                            _b = options.limit, limit = _b === void 0 ? 5 : _b;
                            switch (options.status) {
                                case 'active':
                                    params = { nonce: nonce };
                                    break;
                                case 'executed':
                                    // Need to double limit because some safe Txs are paired.
                                    params = { nonce__lt: nonce, limit: limit * 2 };
                                    break;
                                case 'queued':
                                    // Transaction service returns queued txs by default
                                    params = { limit: limit * 2 };
                                    break;
                                default:
                                    // No status defined, only grab active and below.
                                    params = { limit: limit * 2, nonce__lte: nonce };
                            }
                            _d = (_c = Promise).all;
                            return [4 /*yield*/, (0, transaction_service_1.getSafeTransactionsBySafe)(this.safe, params)];
                        case 1: return [4 /*yield*/, _d.apply(_c, [(_e.sent()).map(function (tx) {
                                    return (0, transaction_service_1.populateDataDecoded)(__assign(__assign({}, tx), { confirmationsRequired: threshold }));
                                })])];
                        case 2:
                            safeTransactions = _e.sent();
                            normalTransactions = [];
                            rejectTransactions = [];
                            pairedSubTxs = {};
                            safeTransactions.forEach(function (tx) {
                                var _a;
                                if (tx.data === null && tx.to === _this.safe) {
                                    rejectTransactions.push(tx);
                                    return;
                                }
                                if (((_a = tx.dataDecoded) === null || _a === void 0 ? void 0 : _a.method) === 'approveHash') {
                                    // Pair approve/reject sub transactions together
                                    // Sub transactions always have an approve, but do not always have a reject
                                    if (Array.isArray(pairedSubTxs[tx.nonce])) {
                                        pairedSubTxs[tx.nonce].push(tx);
                                        return;
                                    }
                                    pairedSubTxs[tx.nonce] = [tx];
                                    return;
                                }
                                normalTransactions.push(tx);
                            });
                            rejectNonces = rejectTransactions.map(function (tx) { return tx.nonce; });
                            subProposals = Object.keys(pairedSubTxs).map(function (subTxNonce) {
                                // subTxPair is an array of length 1 or 2, depending on if there's a reject or not
                                var subTxPair = pairedSubTxs[subTxNonce];
                                if (subTxPair.length === 1) {
                                    return new Proposal_1["default"](_this, nonce, subTxPair[0]);
                                }
                                // If the length is 2, that means there is a paired reject transaction.
                                // The reject transaction is always created after the approve transaction
                                // TODO: I'm actually not sure if that assumption is true anymore.
                                // Because safeTx comes in reverse chron order, subTxPair[1] is the approve, [0] is the reject
                                return new Proposal_1["default"](_this, nonce, subTxPair[1], subTxPair[0]);
                            });
                            normalProposals = normalTransactions.map(function (tx) {
                                // Check to see if there is a corresponding reject nonce.
                                var rejectNonceIndex = rejectNonces.indexOf(tx.nonce);
                                // If there is, we package that together with the regular transaction.
                                if (rejectNonceIndex >= 0) {
                                    return new Proposal_1["default"](_this, nonce, tx, rejectTransactions[rejectNonceIndex]);
                                }
                                // Otherwise, just handle it normally.
                                return new Proposal_1["default"](_this, nonce, tx);
                            });
                            return [2 /*return*/, subProposals
                                    .concat(normalProposals)
                                    .sort(function (a, b) {
                                    // Sort in descending order/reverse chron based on nonce/id.
                                    return b.id - a.id;
                                })
                                    .slice(0, limit)]; // Slice to return the requested limited Proposals.
                    }
                });
            });
        };
        /**
         * Gets a specific proposal by either nonce or the safeTxHash.
         * @param identifier - Can be either the proposal id/nonce (preferred), or the safeTxHash
         */
        this.getProposal = function (identifier) { return __awaiter(_this, void 0, void 0, function () {
            var nonce, safeTransaction, safeTransactions;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(typeof identifier === 'number')) return [3 /*break*/, 1];
                        nonce = identifier;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, (0, transaction_service_1.getSafeTransactionByHash)(identifier)];
                    case 2:
                        safeTransaction = _a.sent();
                        nonce = safeTransaction.nonce;
                        _a.label = 3;
                    case 3: return [4 /*yield*/, (0, transaction_service_1.getSafeTransactionsBySafe)(this.safe, {
                            nonce: nonce
                        })];
                    case 4:
                        safeTransactions = _a.sent();
                        if (safeTransactions.length === 0)
                            throw new Error('Could not find a related safe transaction');
                        if (safeTransactions.length === 1)
                            return [2 /*return*/, new Proposal_1["default"](this, this.nonce, safeTransactions[0])];
                        if (safeTransactions.length === 2)
                            return [2 /*return*/, new Proposal_1["default"](this, this.nonce, safeTransactions[1], safeTransactions[0])];
                        throw new Error('Unexpected number of safe transactions found');
                }
            });
        }); };
        /**
         * Returns an array of this pod's super pods, i.e., pods that this pod is a member of
         */
        this.getSuperPods = function () { return __awaiter(_this, void 0, void 0, function () {
            var userPodIds, _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (this.superPods)
                            return [2 /*return*/, this.superPods];
                        return [4 /*yield*/, (0, subgraph_1.fetchUserPodIds)(this.safe)];
                    case 1:
                        userPodIds = _b.sent();
                        _a = this;
                        return [4 /*yield*/, Promise.all(userPodIds.map(function (podId) { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                                return [2 /*return*/, new Pod(podId)];
                            }); }); }))];
                    case 2:
                        _a.superPods = _b.sent();
                        return [2 /*return*/, this.superPods];
                }
            });
        }); };
        /**
         * Returns an array of all active super proposals, i.e., active proposals of any super pods
         */
        this.getSuperProposals = function () { return __awaiter(_this, void 0, void 0, function () {
            var superPods, superProposals;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getSuperPods()];
                    case 1:
                        superPods = _a.sent();
                        if (superPods.length === 0)
                            return [2 /*return*/, []];
                        return [4 /*yield*/, Promise.all(superPods.map(function (pod) { return __awaiter(_this, void 0, void 0, function () {
                                var activeProposal;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, pod.getProposals({ status: 'active' })];
                                        case 1:
                                            activeProposal = (_a.sent())[0];
                                            return [2 /*return*/, activeProposal];
                                    }
                                });
                            }); }))];
                    case 2:
                        superProposals = (_a.sent()).filter(function (x) { return x; });
                        return [2 /*return*/, superProposals];
                }
            });
        }); };
        /**
         * Returns of list of all member addresses.
         * Members include member pods and member EOAs
         */
        this.getMembers = function () { return __awaiter(_this, void 0, void 0, function () {
            var users;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.members)
                            return [2 /*return*/, this.members];
                        return [4 /*yield*/, (0, subgraph_1.fetchPodUsers)(this.id)];
                    case 1:
                        users = _a.sent();
                        // Checksum all addresses.
                        this.members = users.length > 0 ? users.map(function (user) { return ethers_1.ethers.utils.getAddress(user); }) : [];
                        return [2 /*return*/, this.members];
                }
            });
        }); };
        /**
         * Returns list of all member EOAs, not including any smart contract/pod members.
         */
        this.getMemberEOAs = function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.memberEOAs)
                            return [2 /*return*/, this.memberEOAs];
                        return [4 /*yield*/, this.populateMembers()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.memberEOAs];
                }
            });
        }); };
        /**
         * Returns Pod objects of all member pods.
         */
        this.getMemberPods = function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.memberPods)
                            return [2 /*return*/, this.memberPods];
                        return [4 /*yield*/, this.populateMembers()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.memberPods];
                }
            });
        }); };
        /**
         * Calls a Pod method via a persona.
         * The persona object can be fetched by using `Pod.getPersonas(address)`
         * E.g.,
         * ```
         * callAsPersona(
         *  pod.mintMember,
         *  [newMember],
         *  { type: 'admin', address: userAddress },
         *  signer,
         * )
         * ```
         * The sender must be a signer in the admin case, or the address of the sender.
         * The sender must be a member of the relevant pod.
         *
         * @param method
         * @param args
         * @param persona
         * @param sender
         * @returns
         */
        this.callAsPersona = function (method, args, persona, sender) { return __awaiter(_this, void 0, void 0, function () {
            var senderAddress, _a, _b, _c, adminPod, result, _d, _e, err_1, subPod, isSubPodMember, result, _f, _g, _h, err_2;
            return __generator(this, function (_j) {
                switch (_j.label) {
                    case 0:
                        if (!(sender instanceof ethers_1.ethers.Signer)) return [3 /*break*/, 2];
                        return [4 /*yield*/, sender.getAddress()];
                    case 1:
                        _a = _j.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = sender;
                        _j.label = 3;
                    case 3:
                        senderAddress = _a;
                        _b = persona.type;
                        switch (_b) {
                            case 'admin': return [3 /*break*/, 4];
                            case 'member': return [3 /*break*/, 5];
                            case 'adminPodMember': return [3 /*break*/, 7];
                            case 'subPodMember': return [3 /*break*/, 16];
                        }
                        return [3 /*break*/, 25];
                    case 4:
                        // for the admin case, the sender must be a signer
                        if (!sender)
                            throw new Error("Expected sender to be signer, but received ".concat(sender));
                        args.push(sender);
                        // admin personas do not require proposals
                        return [2 /*return*/, method.apply(this, args)];
                    case 5:
                        _c = this.propose;
                        return [4 /*yield*/, method.apply(this, args)];
                    case 6: 
                    // member personas require proposals to be created
                    return [2 /*return*/, _c.apply(this, [_j.sent(), persona.address])];
                    case 7:
                        adminPod = void 0;
                        if (!this.adminPod) return [3 /*break*/, 8];
                        adminPod = this.adminPod;
                        return [3 /*break*/, 10];
                    case 8: return [4 /*yield*/, new Pod(this.admin)];
                    case 9:
                        // create a new admin pod object with admin address
                        adminPod = _j.sent();
                        this.adminPod = adminPod;
                        _j.label = 10;
                    case 10:
                        result = void 0;
                        _j.label = 11;
                    case 11:
                        _j.trys.push([11, 14, , 15]);
                        _e = (_d = adminPod).propose;
                        return [4 /*yield*/, method.apply(this, args)];
                    case 12: return [4 /*yield*/, _e.apply(_d, [_j.sent(), senderAddress])];
                    case 13:
                        // admin pods require proposal creation
                        result = _j.sent();
                        return [3 /*break*/, 15];
                    case 14:
                        err_1 = _j.sent();
                        // Make the error message more specific
                        if (err_1.message.includes('Sender must be a member of')) {
                            throw new Error('Sender must be a member of the admin pod');
                        }
                        return [3 /*break*/, 15];
                    case 15: return [2 /*return*/, result];
                    case 16: return [4 /*yield*/, new Pod(persona.address)];
                    case 17:
                        subPod = _j.sent();
                        return [4 /*yield*/, this.isMember(subPod.safe)];
                    case 18:
                        isSubPodMember = _j.sent();
                        if (!isSubPodMember)
                            throw new Error('Sub pod is not a member of this pod');
                        result = void 0;
                        _j.label = 19;
                    case 19:
                        _j.trys.push([19, 23, , 24]);
                        _g = (_f = subPod).propose;
                        _h = this.propose;
                        return [4 /*yield*/, method.apply(this, args)];
                    case 20: return [4 /*yield*/, _h.apply(this, [_j.sent(), subPod.safe])];
                    case 21: return [4 /*yield*/, _g.apply(_f, [_j.sent(), senderAddress])];
                    case 22:
                        // sub pods require proposal creation
                        result = _j.sent();
                        return [3 /*break*/, 24];
                    case 23:
                        err_2 = _j.sent();
                        // Make the error message more specific
                        if (err_2.message.includes('Sender must be a member of')) {
                            throw new Error('Sender must be a member of the sub pod');
                        }
                        return [3 /*break*/, 24];
                    case 24: return [2 /*return*/, result];
                    case 25: throw new Error("".concat(persona.type, " is not a valid persona"));
                }
            });
        }); };
        /**
         * Checks if user is a member of this pod
         * @param address
         */
        this.isMember = function (address) { return __awaiter(_this, void 0, void 0, function () {
            var checkedAddress;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedAddress = (0, utils_1.checkAddress)(address);
                        if (!!this.members) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.getMembers()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/, this.members.includes(checkedAddress)];
                }
            });
        }); };
        /**
         * Checks if user is admin of this pod
         * @param address
         */
        this.isAdmin = function (address) {
            var checkedAddress = (0, utils_1.checkAddress)(address);
            return checkedAddress === _this.admin;
        };
        /**
         * Checks if given address is a member of the admin pod (if there is one)
         * Returns false if there is no admin pod.
         */
        this.isAdminPodMember = function (address) { return __awaiter(_this, void 0, void 0, function () {
            var checkedAddress, adminPod;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedAddress = (0, utils_1.checkAddress)(address);
                        if (!this.admin)
                            return [2 /*return*/, false];
                        if (!this.adminPod) return [3 /*break*/, 1];
                        adminPod = this.adminPod;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, new Pod(this.admin)];
                    case 2:
                        adminPod = _a.sent();
                        this.adminPod = adminPod;
                        _a.label = 3;
                    case 3:
                        if (!adminPod)
                            return [2 /*return*/, false];
                        return [2 /*return*/, adminPod.isMember(checkedAddress)];
                }
            });
        }); };
        /**
         * Checks if given address is a member of any subpods.
         * Returns false if the user is a member of **this** pod, but not any sub pods
         *
         * @param address
         */
        this.isSubPodMember = function (address) { return __awaiter(_this, void 0, void 0, function () {
            var checkedAddress, results;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedAddress = (0, utils_1.checkAddress)(address);
                        if (!!this.memberPods) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.populateMembers()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [4 /*yield*/, Promise.all(this.memberPods.map(function (pod) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                return [2 /*return*/, pod.isMember(checkedAddress)];
                            });
                        }); }))];
                    case 3:
                        results = _a.sent();
                        return [2 /*return*/, results.includes(true)];
                }
            });
        }); };
        /**
         * Returns all sub pods of this pod that an address is the member of.
         * @param address
         */
        this.getSubPodsByMember = function (address) { return __awaiter(_this, void 0, void 0, function () {
            var checkedAddress, results;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedAddress = (0, utils_1.checkAddress)(address);
                        if (!!this.memberPods) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.populateMembers()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [4 /*yield*/, Promise.all(this.memberPods.map(function (pod) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, pod.isMember(checkedAddress)];
                                    case 1: return [2 /*return*/, (_a.sent()) ? pod : null];
                                }
                            });
                        }); }))];
                    case 3:
                        results = _a.sent();
                        return [2 /*return*/, results.filter(function (x) { return !!x; })];
                }
            });
        }); };
        /**
         * Fetches an external pod (i.e., a sub or admin pod) and performs checks.
         * Mostly used in context of creating sub/super proposals.
         * @ignore
         * @param podIdentifier - Pod safe, ID, or the Pod object itself
         * @param relationship - 'admin' or 'sub'
         * @param signerAddress - Address of signer
         * @returns
         */
        this.getExternalPod = function (podIdentifier, relationship, signerAddress) { return __awaiter(_this, void 0, void 0, function () {
            var externalPod;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(podIdentifier instanceof Pod)) return [3 /*break*/, 1];
                        externalPod = podIdentifier;
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, new Pod(podIdentifier)];
                    case 2:
                        externalPod = _a.sent();
                        _a.label = 3;
                    case 3:
                        if (!externalPod)
                            throw new Error("Could not find a pod with identifier ".concat(podIdentifier));
                        if (!(relationship === 'admin')) return [3 /*break*/, 4];
                        if (!this.isAdmin(externalPod.safe)) {
                            throw new Error("Pod ".concat(externalPod.ensName, " must be the admin of this pod to make proposals"));
                        }
                        return [3 /*break*/, 6];
                    case 4:
                        if (!(relationship === 'sub')) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.isMember(externalPod.safe)];
                    case 5:
                        if (!(_a.sent())) {
                            throw new Error("Pod ".concat(externalPod.ensName, " must be a subpod of this pod to make proposals"));
                        }
                        _a.label = 6;
                    case 6: return [4 /*yield*/, externalPod.isMember(signerAddress)];
                    case 7:
                        if (!(_a.sent()))
                            throw new Error("Signer ".concat(signerAddress, " was not a member of the ").concat(relationship, " pod ").concat(externalPod.ensName));
                        return [2 /*return*/, externalPod];
                }
            });
        }); };
        /**
         * Creates a proposal on the pod.
         *
         * If the proposal parameter is in the { data, to } format, this will create a proposal to execute
         * an arbitrary smart contract function on this pod.
         *
         * The `data` parameter is unsigned encoded function data. You can generate this function data a number of ways,
         * our recommended way would be through ethers.Interface.encodeFunctionData. The `to` parameter should be the smart contract
         * that the function being called belongs to.
         *
         * The pod object can also populate these transactions by using the supplied methods with no signer (i.e., `pod.mint()`)
         *
         * To create a proposal on the current pod, you can chain the method and the propose method like so:
         *
         * ```js
         * await pod.propose(await pod.mint(newMember), podMember);
         * ```
         *
         * To create a proposal on the admin pod to mint to a managed pod, you can call the managed pod's method:
         * Note that the pod you call the functions on is important: it determines which pod you are attempting to mint to.
         *
         * ```js
         * // It is important that you call mint() from managedPod here
         * await adminPod.propose(await managedPod.mint(newMember), adminPodMember);
         * ```
         *
         * In order to create a sub proposal of an existing proposal, you can pass the proposal object to the sub pod's propose method:
         *
         * ```
         * const [superProposal] = await superPod.getProposals();
         * // This creates a sub proposal to approve the super proposal.
         * await subPod.propose(superProposal, signer);
         *
         * // The propose function also returns a Propose object, so you can chain `propose` like so:
         * await subPod.propose(
         *   await superPod.propose(
         *     await superPod.mint(newMember),
         *     superPodMember,
         *   ),
         *   subPodMember,
         * );
         * ```
         *
         * The sender parameter should be the address the proposal is being sent from. This should be a member
         * of the pod for creating a proposal, or a member of a sub pod to create sub proposals.
         *
         * @param proposal
         * @param sender - Address of sender
         * @param opts.nonce - Optional nonce. This will create a proposal with the given nonce.
         * @returns
         */
        this.propose = function (
        // TODO: We don't actually accept a TransactionResponse, this is just to bypass typescript.
        proposal, sender, opts) {
            if (opts === void 0) { opts = {}; }
            return __awaiter(_this, void 0, void 0, function () {
                var nonce, safeTransaction, safeTxHash, err_3, to, data, _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            nonce = opts.nonce;
                            if (!(proposal instanceof Proposal_1["default"])) return [3 /*break*/, 6];
                            return [4 /*yield*/, this.isMember(sender)];
                        case 1:
                            if (!(_b.sent())) {
                                throw new Error('Sender must be part of this pod to create a proposal');
                            }
                            safeTxHash = proposal.safeTransaction.safeTxHash;
                            _b.label = 2;
                        case 2:
                            _b.trys.push([2, 4, , 5]);
                            return [4 /*yield*/, (0, create_safe_transaction_1.createSafeTransaction)({
                                    sender: sender,
                                    to: proposal.pod.safe,
                                    data: (0, utils_1.encodeFunctionData)('GnosisSafe', 'approveHash', [safeTxHash]),
                                    safe: this.safe
                                })];
                        case 3:
                            safeTransaction = _b.sent();
                            return [3 /*break*/, 5];
                        case 4:
                            err_3 = _b.sent();
                            if (err_3.response.data.message === 'Gas estimation failed') {
                                throw new Error('Gas estimation failed (this is often a revert error)');
                            }
                            throw new Error(err_3);
                        case 5: return [3 /*break*/, 11];
                        case 6:
                            to = proposal.to, data = proposal.data;
                            return [4 /*yield*/, this.isMember(sender)];
                        case 7:
                            _a = (_b.sent());
                            if (_a) return [3 /*break*/, 9];
                            return [4 /*yield*/, this.isSubPodMember(sender)];
                        case 8:
                            _a = (_b.sent());
                            _b.label = 9;
                        case 9:
                            if (!(_a)) {
                                throw new Error('Sender must be a member of this pod or one of its sub pods');
                            }
                            return [4 /*yield*/, (0, create_safe_transaction_1.createSafeTransaction)({
                                    sender: sender,
                                    safe: this.safe,
                                    to: to,
                                    data: data,
                                    nonce: nonce || null
                                })];
                        case 10:
                            safeTransaction = _b.sent();
                            _b.label = 11;
                        case 11: return [2 /*return*/, new Proposal_1["default"](this, this.nonce, safeTransaction)];
                    }
                });
            });
        };
        /**
         * Mints member(s) to this pod, or returns an unsigned transaction to do so.
         * @param newMember - Can be a single address or an array of addresses
         * @param signer - If a signer is provided, then the tx will execute. Otherwise, an unsigned transaction will be returned.
         * @throws if signer is not admin
         */
        this.mintMember = function (newMember, signer) { return __awaiter(_this, void 0, void 0, function () {
            var signerAddress, MemberToken, err_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!signer) return [3 /*break*/, 2];
                        return [4 /*yield*/, signer.getAddress()];
                    case 1:
                        signerAddress = _a.sent();
                        if (!this.isAdmin(signerAddress))
                            throw new Error('Signer was not admin');
                        _a.label = 2;
                    case 2:
                        MemberToken = (0, utils_1.getMetropolisContract)('MemberToken', signer);
                        if (!Array.isArray(newMember)) return [3 /*break*/, 4];
                        newMember.forEach(utils_1.checkAddress);
                        if (signer)
                            return [2 /*return*/, MemberToken.mintSingleBatch(newMember, this.id, ethers_1.ethers.constants.HashZero)];
                        return [4 /*yield*/, MemberToken.populateTransaction.mintSingleBatch(newMember, this.id, ethers_1.ethers.constants.HashZero)];
                    case 3: return [2 /*return*/, (_a.sent())];
                    case 4:
                        // Single member.
                        (0, utils_1.checkAddress)(newMember);
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        if (signer)
                            return [2 /*return*/, MemberToken.mint(newMember, this.id, ethers_1.ethers.constants.HashZero)];
                        return [4 /*yield*/, MemberToken.populateTransaction.mint(newMember, this.id, ethers_1.ethers.constants.HashZero)];
                    case 6: return [2 /*return*/, (_a.sent())];
                    case 7:
                        err_4 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_4)];
                    case 8: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Burns member(s) from this pod, or returns an unsigned transaction to do so.
         * @param memberToBurn - Can be a single address or an array of addresses
         * @param signer - If a signer is provided, then the tx will execute. Otherwise, an unsigned transaction will be returned.
         * @throws If signer is not admin
         */
        this.burnMember = function (memberToBurn, signer) { return __awaiter(_this, void 0, void 0, function () {
            var signerAddress, MemberToken, err_5, MemberToken, err_6;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!signer) return [3 /*break*/, 2];
                        return [4 /*yield*/, signer.getAddress()];
                    case 1:
                        signerAddress = _a.sent();
                        if (!this.isAdmin(signerAddress))
                            throw new Error('Signer was not admin');
                        _a.label = 2;
                    case 2:
                        if (!Array.isArray(memberToBurn)) return [3 /*break*/, 6];
                        memberToBurn.forEach(utils_1.checkAddress);
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 6]);
                        MemberToken = (0, utils_1.getMetropolisContract)('MemberToken', signer);
                        if (signer)
                            return [2 /*return*/, MemberToken.burnSingleBatch(memberToBurn, this.id)];
                        return [4 /*yield*/, MemberToken.populateTransaction.burnSingleBatch(memberToBurn, this.id)];
                    case 4: return [2 /*return*/, (_a.sent())];
                    case 5:
                        err_5 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_5)];
                    case 6:
                        // Burn single
                        (0, utils_1.checkAddress)(memberToBurn);
                        _a.label = 7;
                    case 7:
                        _a.trys.push([7, 9, , 10]);
                        MemberToken = (0, utils_1.getMetropolisContract)('MemberToken', signer);
                        if (signer)
                            return [2 /*return*/, MemberToken.burn(memberToBurn, this.id)];
                        return [4 /*yield*/, MemberToken.populateTransaction.burn(memberToBurn, this.id)];
                    case 8: return [2 /*return*/, (_a.sent())];
                    case 9:
                        err_6 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_6)];
                    case 10: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Mints and burns members simultaneously.
         */
        this.batchMintAndBurn = function (mintMembers, burnMembers, signer) { return __awaiter(_this, void 0, void 0, function () {
            var signerAddress, controllerAbi, Controller, err_7;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        mintMembers.forEach(utils_1.checkAddress);
                        burnMembers.forEach(utils_1.checkAddress);
                        if (!signer) return [3 /*break*/, 2];
                        return [4 /*yield*/, signer.getAddress()];
                    case 1:
                        signerAddress = _a.sent();
                        if (!this.isAdmin(signerAddress))
                            throw new Error('Signer was not admin');
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        controllerAbi = (0, contracts_1.getControllerByAddress)(this.controller, config_1.config.network).abi;
                        Controller = new ethers_1.ethers.Contract(this.controller, controllerAbi, signer);
                        if (signer)
                            return [2 /*return*/, Controller.batchMintAndBurn(this.id, mintMembers, burnMembers)];
                        return [4 /*yield*/, Controller.populateTransaction.batchMintAndBurn(this.id, mintMembers, burnMembers)];
                    case 3: return [2 /*return*/, (_a.sent())];
                    case 4:
                        err_7 = _a.sent();
                        if (err_7.message.includes('batchMintAndBurn is not a function')) {
                            throw new Error('batchMintAndBurn not found, you may need to upgrade to the latest Controller version');
                        }
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_7)];
                    case 5: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Transfers a membership. If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
         *
         * @param toAddress - Address that will receive membership
         * @param fromAddress - Address that is giving up membership
         * @param signer - If a signer is provided, then the tx will execute. Otherwise, an unsigned transaction will be returned.
         * @throws If toAddress is already a member
         * @throws if fromAddress is not a member
         * @throws If provided signer is not the fromAddress
         */
        this.transferMembership = function (fromAddress, toAddress, signer) { return __awaiter(_this, void 0, void 0, function () {
            var checkedFrom, checkedTo, signerAddress, MemberToken, err_8;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedFrom = (0, utils_1.checkAddress)(fromAddress);
                        return [4 /*yield*/, this.isMember(fromAddress)];
                    case 1:
                        if (!(_a.sent())) {
                            throw new Error("Address ".concat(fromAddress, " was not a member of this pod"));
                        }
                        checkedTo = (0, utils_1.checkAddress)(toAddress);
                        return [4 /*yield*/, this.isMember(checkedTo)];
                    case 2:
                        if (_a.sent()) {
                            throw new Error("Address ".concat(checkedTo, " is already a member of this pod"));
                        }
                        if (!signer) return [3 /*break*/, 5];
                        return [4 /*yield*/, signer.getAddress()];
                    case 3:
                        signerAddress = _a.sent();
                        if (checkedFrom !== signerAddress)
                            throw new Error('Signer did not match the from address');
                        return [4 /*yield*/, this.isMember(signerAddress)];
                    case 4:
                        if (!(_a.sent())) {
                            throw new Error("Signer ".concat(signerAddress, " is not a member of this pod"));
                        }
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        MemberToken = (0, utils_1.getMetropolisContract)('MemberToken', signer);
                        if (signer) {
                            return [2 /*return*/, MemberToken.safeTransferFrom(checkedFrom, checkedTo, this.id, 1, ethers_1.ethers.constants.HashZero)];
                        }
                        return [4 /*yield*/, MemberToken.populateTransaction.safeTransferFrom(checkedFrom, checkedTo, this.id, 1, ethers_1.ethers.constants.HashZero)];
                    case 6: return [2 /*return*/, (_a.sent())];
                    case 7:
                        err_8 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_8)];
                    case 8: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Changes the threshold for the safe associated with this pod.
         * If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
         * @param newThreshold - new threshold
         * @param signer - Signer of admin
         * @throws If signer is not admin
         * @returns
         */
        this.changeThreshold = function (newThreshold, signer) { return __awaiter(_this, void 0, void 0, function () {
            var signerAddress, GnosisSafe, err_9;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!signer) return [3 /*break*/, 2];
                        return [4 /*yield*/, signer.getAddress()];
                    case 1:
                        signerAddress = _a.sent();
                        if (!this.isAdmin(signerAddress))
                            throw new Error('Signer was not the admin of this pod');
                        _a.label = 2;
                    case 2:
                        if (newThreshold === this.threshold)
                            throw new Error("Current threshold is already ".concat(newThreshold));
                        GnosisSafe = (0, utils_1.getGnosisSafeContract)(this.safe, signer);
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 6]);
                        if (signer)
                            return [2 /*return*/, GnosisSafe.changeThreshold(newThreshold)];
                        return [4 /*yield*/, GnosisSafe.populateTransaction.changeThreshold(newThreshold)];
                    case 4: return [2 /*return*/, (_a.sent())];
                    case 5:
                        err_9 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_9)];
                    case 6: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Transfers admin role from signer's account to addressToTransferTo
         * If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
         * @param addressToTransferTo - Address that will receive admin role
         * @param signer - Signer of admin
         * @throws If signer is not admin
         */
        this.transferAdmin = function (addressToTransferTo, signer) { return __awaiter(_this, void 0, void 0, function () {
            var checkedAddress, signerAddress, controllerAbi, Controller, err_10;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkedAddress = (0, utils_1.checkAddress)(addressToTransferTo);
                        if (!signer) return [3 /*break*/, 2];
                        return [4 /*yield*/, signer.getAddress()];
                    case 1:
                        signerAddress = _a.sent();
                        if (!this.isAdmin(signerAddress))
                            throw new Error('Signer was not the admin of this pod');
                        _a.label = 2;
                    case 2:
                        controllerAbi = (0, contracts_1.getControllerByAddress)(this.controller, config_1.config.network).abi;
                        Controller = new ethers_1.ethers.Contract(this.controller, controllerAbi, signer);
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 6]);
                        if (signer)
                            return [2 /*return*/, Controller.updatePodAdmin(this.id, checkedAddress)];
                        return [4 /*yield*/, Controller.populateTransaction.updatePodAdmin(this.id, checkedAddress)];
                    case 4: return [2 /*return*/, (_a.sent())];
                    case 5:
                        err_10 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_10)];
                    case 6: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Migrates the pod to the latest version. Signer must be the admin of pod.
         * If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
         * @param signer - Signer of pod admin
         * @throws If signer is not pod admin
         */
        this.migratePodToLatest = function (signer) { return __awaiter(_this, void 0, void 0, function () {
            var newController, oldControllerDeployment, OldController, previousModule, err_11;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        newController = (0, contracts_1.getDeployment)('ControllerLatest', config_1.config.network);
                        oldControllerDeployment = (0, contracts_1.getControllerByAddress)(this.controller, config_1.config.network);
                        OldController = new ethers_1.ethers.Contract(oldControllerDeployment.address, oldControllerDeployment.abi, signer);
                        return [4 /*yield*/, (0, utils_1.getPreviousModule)(this.safe, oldControllerDeployment.address, newController.address)];
                    case 1:
                        previousModule = _a.sent();
                        // If the previous module is the sentinel, then we're going to add the new controller
                        // before removing the old controller, therefore the previous module will be the
                        // new controller we just added, because safe modules are added reverse-chronologically.
                        if (previousModule === '0x0000000000000000000000000000000000000001') {
                            previousModule = newController.address;
                        }
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        if (signer)
                            return [2 /*return*/, OldController.migratePodController(this.id, newController.address, previousModule)];
                        return [4 /*yield*/, OldController.populateTransaction.migratePodController(this.id, newController.address, previousModule)];
                    case 3: return [2 /*return*/, (_a.sent())];
                    case 4:
                        err_11 = _a.sent();
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_11)];
                    case 5: return [2 /*return*/];
                }
            });
        }); };
        /**
         *
         */
        this.isPodifyInProgress = function () { return __awaiter(_this, void 0, void 0, function () {
            var proposals, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getProposals()];
                    case 1:
                        proposals = _a.sent();
                        result = false;
                        proposals.forEach(function (proposal) {
                            if (proposal.status === ('passed' || 'rejected'))
                                return;
                            if (proposal.method === 'enableModule') {
                                try {
                                    // This throws if the provided address is not one of our controllers.
                                    (0, contracts_1.getControllerByAddress)(proposal.parameters[0].value, config_1.config.network);
                                    // Set to true if it doesn't throw.
                                    result = true;
                                }
                                catch (_a) {
                                    // Do nothing.
                                }
                            }
                        });
                        return [2 /*return*/, result];
                }
            });
        }); };
        /**
         * Ejects a safe from the Orca ecosystem.
         * This zeroes out all ENS + Controller data, removes the Orca module, and burns the pod's MemberTokens
         * If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
         *
         * This function can also clean up data for safes that have already removed the Orca module,
         * but note that the reverse resolver must be zeroed out by the safe manually in this case.
         */
        this.ejectSafe = function (signer) { return __awaiter(_this, void 0, void 0, function () {
            var previousModule, controllerDeployment, Controller, err_12;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (0, utils_1.getPreviousModule)(this.safe, this.controller)];
                    case 1:
                        previousModule = _a.sent();
                        controllerDeployment = (0, contracts_1.getControllerByAddress)(this.controller, config_1.config.network);
                        Controller = new ethers_1.ethers.Contract(controllerDeployment.address, controllerDeployment.abi, signer);
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        if (signer)
                            return [2 /*return*/, Controller.ejectSafe(this.id, (0, ensjs_1.labelhash)(this.ensName.split('.')[0]), previousModule)];
                        return [4 /*yield*/, Controller.populateTransaction.ejectSafe(this.id, (0, ensjs_1.labelhash)(this.ensName.split('.')[0]), previousModule)];
                    case 3: return [2 /*return*/, (_a.sent())];
                    case 4:
                        err_12 = _a.sent();
                        if (err_12.message.includes('ejectSafe is not a function')) {
                            throw new Error('ejectSafe not found, you may need to upgrade to the latest Controller version');
                        }
                        return [2 /*return*/, (0, utils_1.handleEthersError)(err_12)];
                    case 5: return [2 /*return*/];
                }
            });
        }); };
        /**
         * Creates a reject proposal at a given nonce, mostly used to un-stuck the transaction queue
         * @param nonce - nonce to create the reject transaction at
         * @param signer - Signer or address of pod member
         */
        this.createRejectProposal = function (nonce, signer) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (0, transaction_service_1.createRejectTransaction)({
                            safe: this.safe,
                            to: this.safe,
                            nonce: nonce,
                            confirmationsRequired: this.threshold
                        }, signer)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); };
        var network = config_1.config.network;
        // This is a kind of hacky way to go about an async constructor.
        // It works, typescript just doesn't like it.
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        return (function () { return __awaiter(_this, void 0, void 0, function () {
            var podId, safe, Controller, Name, fetchedAdmin, fetchers, fetched, err_13, baseUrl, paddedId, metadata, _a, imageUrl, imageNoTextUrl;
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _c.trys.push([0, 6, , 7]);
                        fetchers = void 0;
                        if (!(typeof identifier === 'string')) return [3 /*break*/, 2];
                        return [4 /*yield*/, (0, fetchers_1.getPodFetchersByAddressOrEns)(identifier)];
                    case 1:
                        fetchers = _c.sent();
                        return [3 /*break*/, 4];
                    case 2:
                        if (!(typeof identifier === 'number')) return [3 /*break*/, 4];
                        return [4 /*yield*/, (0, fetchers_1.getPodFetchersById)(identifier)];
                    case 3:
                        fetchers = _c.sent();
                        _c.label = 4;
                    case 4:
                        podId = fetchers.podId;
                        safe = fetchers.Safe.address;
                        return [4 /*yield*/, Promise.all([
                                fetchers.Safe.nonce(),
                                fetchers.Safe.getThreshold(),
                                fetchers.Controller.podAdmin(podId),
                            ])];
                    case 5:
                        fetched = _c.sent();
                        this.nonce = fetched[0].toNumber();
                        this.threshold = fetched[1].toNumber();
                        fetchedAdmin = fetched[2];
                        Controller = fetchers.Controller;
                        Name = fetchers.Name;
                        return [3 /*break*/, 7];
                    case 6:
                        err_13 = _c.sent();
                        if (err_13.message.includes('invalid address')) {
                            throw new TypeError("Non-address string passed to Pod constructor: '".concat(identifier, "'"));
                        }
                        return [2 /*return*/, null];
                    case 7:
                        this.controller = Controller.address;
                        this.admin = fetchedAdmin === ethers_1.ethers.constants.AddressZero ? null : fetchedAdmin;
                        this.id = podId;
                        this.safe = safe;
                        this.address = this.safe;
                        this.ensName = Name.name;
                        this.podName = ((_b = this.ensName) === null || _b === void 0 ? void 0 : _b.includes('.pod')) ? this.ensName.split('.')[0] : this.ensName;
                        baseUrl = "https://metropolis-nft.herokuapp.com/assets/".concat(network, "/");
                        paddedId = podId.toString(16).padStart(64, '0');
                        _c.label = 8;
                    case 8:
                        _c.trys.push([8, 10, , 11]);
                        return [4 /*yield*/, axios_1["default"].get("".concat(baseUrl).concat(paddedId, ".json"))];
                    case 9:
                        metadata = (_c.sent()).data;
                        this.description = metadata.description || null;
                        return [3 /*break*/, 11];
                    case 10:
                        _a = _c.sent();
                        this.description = null;
                        return [3 /*break*/, 11];
                    case 11:
                        imageUrl = "".concat(baseUrl).concat(paddedId, "/image");
                        imageNoTextUrl = "".concat(baseUrl).concat(paddedId, "/image-no-text");
                        this.imageUrl = imageUrl;
                        this.imageNoTextUrl = imageNoTextUrl;
                        return [2 /*return*/, this];
                }
            });
        }); })();
    }
    /**
     * @ignore
     * Populates the memberEOAs and memberPods fields.
     * The process for fetching either of these fields is the same.
     */
    Pod.prototype.populateMembers = function () {
        return __awaiter(this, void 0, void 0, function () {
            var EOAs, memberPods;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.members) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.getMembers()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        EOAs = [];
                        return [4 /*yield*/, Promise.all(this.members.map(function (member) { return __awaiter(_this, void 0, void 0, function () {
                                var pod;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, new Pod(member)];
                                        case 1:
                                            pod = _a.sent();
                                            // If pod is null, it's an EOA.
                                            if (!pod) {
                                                EOAs.push(member);
                                                return [2 /*return*/, pod];
                                            }
                                            return [2 /*return*/, pod];
                                    }
                                });
                            }); }))];
                    case 3:
                        memberPods = (_a.sent()).filter(function (x) { return !!x; });
                        this.memberEOAs = EOAs;
                        this.memberPods = memberPods;
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Fetches all personas for a given address related to this pod.
     * All personas return as an object indicating the type of the persona and the address of the persona.
     * For members and admins, the persona address is the user's address.
     * For admin pods and sub pods, the persona address is the pod address.
     * @param address
     */
    Pod.prototype.getPersonas = function (address) {
        // Function declared separately out for test/mocking purposes.
        return (0, pod_utils_1.getPersonas)(this, address);
    };
    return Pod;
}());
exports["default"] = Pod;
