"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var ethers_1 = require("ethers");
var utils_1 = require("./lib/utils");
var Pod_1 = __importDefault(require("./Pod"));
/**
 * Batch transfers pod memberships. If a signer is provided, it will execute the transaction. Otherwise it will return the unsigned tx.
 *
 * @param fromAddress - Address that is giving up membership
 * @param toAddress - Address that will receive membership
 * @param podIds - Array of pod IDs for which the toAddress will receive membership
 * @param signer - If a signer is provided, then the tx will execute. Otherwise, an unsigned transaction will be returned.
 * @throws If toAddress is already a member
 * @throws if fromAddress is not a member
 * @throws If provided signer is not the fromAddress
 */
function batchTransferMembership(fromAddress, toAddress, podIds, signer) {
    return __awaiter(this, void 0, void 0, function () {
        var pods, checkedFrom, checkedTo, isToAddressPodMember, signerAddress, isSignerPodMember, isSameVersion, MemberToken, valueArray, err_1;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, Promise.all(podIds.map(function (podId) { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                        return [2 /*return*/, new Pod_1["default"](podId)];
                    }); }); }))];
                case 1:
                    pods = _a.sent();
                    checkedFrom = (0, utils_1.checkAddress)(fromAddress);
                    checkedTo = (0, utils_1.checkAddress)(toAddress);
                    return [4 /*yield*/, (0, utils_1.isPodMember)(pods, checkedTo)];
                case 2:
                    isToAddressPodMember = _a.sent();
                    // if a user is already a pod member, throw an error
                    if (isToAddressPodMember) {
                        throw new Error("Signer ".concat(checkedTo, " is already a member of this pod"));
                    }
                    if (!signer) return [3 /*break*/, 5];
                    return [4 /*yield*/, signer.getAddress()];
                case 3:
                    signerAddress = _a.sent();
                    if (checkedFrom !== signerAddress)
                        throw new Error('Signer did not match the from address');
                    return [4 /*yield*/, (0, utils_1.isPodMember)(pods, signerAddress)];
                case 4:
                    isSignerPodMember = _a.sent();
                    // if the user isn't a member of all pods, throw an error
                    if (!isSignerPodMember) {
                        throw new Error("Signer ".concat(signerAddress, " is not a member of this pod"));
                    }
                    _a.label = 5;
                case 5:
                    isSameVersion = (0, utils_1.getIsSameVersion)(pods.map(function (pod) { return pod.controller; }));
                    if (!isSameVersion) {
                        throw new Error('All pods must be on the same controller version');
                    }
                    _a.label = 6;
                case 6:
                    _a.trys.push([6, 8, , 9]);
                    MemberToken = (0, utils_1.getMetropolisContract)('MemberToken', signer);
                    valueArray = new Array(podIds.length).fill(1);
                    if (signer) {
                        return [2 /*return*/, MemberToken.safeBatchTransferFrom(checkedFrom, checkedTo, podIds, valueArray, ethers_1.ethers.constants.HashZero)];
                    }
                    return [4 /*yield*/, MemberToken.populateTransaction.safeBatchTransferFrom(checkedFrom, checkedTo, podIds, valueArray, ethers_1.ethers.constants.HashZero)];
                case 7: return [2 /*return*/, (_a.sent())];
                case 8:
                    err_1 = _a.sent();
                    return [2 /*return*/, (0, utils_1.handleEthersError)(err_1)];
                case 9: return [2 /*return*/];
            }
        });
    });
}
exports["default"] = batchTransferMembership;
