import Pod from './Pod';
import Proposal from './Proposal';
import batchTransferMembership from './batch';
import { multiPodCreate, podifySafe, enableController, createPod } from './pod-create';
import { Pod as PodType, Proposal as ProposalType, ProposalStatus } from './types';
import { init, config } from './config';
import { fetchPodUsers, fetchUserPodIds, fetchAdminPodIds, customSubgraphQuery } from './lib/services/subgraph';
/**
 * Gets a pod object.
 *
 * @param identifier - Pod ID (as number), safe address, or ENS name
 */
declare function getPod(identifier: string | number): Promise<Pod>;
/**
 * Gets the super proposal that the sub proposal relates to
 * @throws If this proposal is not a sub proposal
 * @throws If the found super proposal does not relate to this sub proposal.
 * @returns
 */
declare function getSuperProposal(proposal: Proposal): Promise<Proposal>;
/**
 * Fetches an array of Pod objects that a user is a member of
 *
 * @param address - user address
 */
declare function getUserPods(address: string): Promise<Pod[]>;
/**
 * Gets an array of Pod objects that a user is the admin of
 * @param address - user address
 */
declare function getAdminPods(address: string): Promise<Pod[]>;
export { init, config, getPod, getSuperProposal, getUserPods, getAdminPods, fetchPodUsers, fetchUserPodIds, fetchAdminPodIds, customSubgraphQuery, multiPodCreate, createPod, podifySafe, enableController, batchTransferMembership, PodType as Pod, ProposalType as Proposal, ProposalStatus, };
