import { ethers } from 'ethers';
import type Pod from '../../Pod';
import { SafeTransaction } from './transaction-service';
/**
 * Gets the nonce of the next transaction for a safe
 * @param safeAddress - safe address
 * @returns Nonce of next transaction
 */
export declare function getNextNonce(safeAddress: string): Promise<number>;
/**
 * Creates safe transaction
 * @param input.safe - Address of Gnosis safe
 * @param input.to - Smart contract address (i.e., MemberToken)
 * @param input.value - Value to send
 * @param input.data - Transaction data
 * @param input.sender - Address that is initiating the transaction
 * @returns {SafeTransaction}
 */
export declare function createSafeTransaction(input: {
    safe: string;
    to: string;
    value?: string;
    data?: string;
    sender: string;
    nonce?: number;
}): Promise<SafeTransaction>;
/**
 * Rejects a super proposal
 *
 * Super proposal rejections, from the sub proposal point of view, are separate approveHash calls that approve
 * a rejection transaction on the super pod
 *
 * @param superProposalTxHash - The transaction hash that identifies the original super proposal (i.e., not the rejection super proposal)
 * @param subProposal - The sub proposal related to the superProposalTxHash
 * @param signer - Signer of sub pod member
 */
export declare function rejectSuperProposal(superProposalTxHash: string, subPod: Pod, signer: ethers.Signer): Promise<void>;
